# fish completion for tt                                   -*- shell-script -*-

function __tt_debug
    set -l file "$BASH_COMP_DEBUG_FILE"
    if test -n "$file"
        echo "$argv" >> $file
    end
end

function __tt_perform_completion
    __tt_debug "Starting __tt_perform_completion"

    # Extract all args except the last one
    set -l args (commandline -opc)
    # Extract the last arg and escape it in case it is a space
    set -l lastArg (string escape -- (commandline -ct))

    __tt_debug "args: $args"
    __tt_debug "last arg: $lastArg"

    # Disable ActiveHelp which is not supported for fish shell
    set -l requestComp "TT_ACTIVE_HELP=0 $args[1] __complete $args[2..-1] $lastArg"

    __tt_debug "Calling $requestComp"
    set -l results (eval $requestComp 2> /dev/null)

    # Some programs may output extra empty lines after the directive.
    # Let's ignore them or else it will break completion.
    # Ref: https://github.com/spf13/cobra/issues/1279
    for line in $results[-1..1]
        if test (string trim -- $line) = ""
            # Found an empty line, remove it
            set results $results[1..-2]
        else
            # Found non-empty line, we have our proper output
            break
        end
    end

    set -l comps $results[1..-2]
    set -l directiveLine $results[-1]

    # For Fish, when completing a flag with an = (e.g., <program> -n=<TAB>)
    # completions must be prefixed with the flag
    set -l flagPrefix (string match -r -- '-.*=' "$lastArg")

    __tt_debug "Comps: $comps"
    __tt_debug "DirectiveLine: $directiveLine"
    __tt_debug "flagPrefix: $flagPrefix"

    for comp in $comps
        printf "%s%s\n" "$flagPrefix" "$comp"
    end

    printf "%s\n" "$directiveLine"
end

# this function limits calls to __tt_perform_completion, by caching the result behind $__tt_perform_completion_once_result
function __tt_perform_completion_once
    __tt_debug "Starting __tt_perform_completion_once"

    if test -n "$__tt_perform_completion_once_result"
        __tt_debug "Seems like a valid result already exists, skipping __tt_perform_completion"
        return 0
    end

    set --global __tt_perform_completion_once_result (__tt_perform_completion)
    if test -z "$__tt_perform_completion_once_result"
        __tt_debug "No completions, probably due to a failure"
        return 1
    end

    __tt_debug "Performed completions and set __tt_perform_completion_once_result"
    return 0
end

# this function is used to clear the $__tt_perform_completion_once_result variable after completions are run
function __tt_clear_perform_completion_once_result
    __tt_debug ""
    __tt_debug "========= clearing previously set __tt_perform_completion_once_result variable =========="
    set --erase __tt_perform_completion_once_result
    __tt_debug "Successfully erased the variable __tt_perform_completion_once_result"
end

function __tt_requires_order_preservation
    __tt_debug ""
    __tt_debug "========= checking if order preservation is required =========="

    __tt_perform_completion_once
    if test -z "$__tt_perform_completion_once_result"
        __tt_debug "Error determining if order preservation is required"
        return 1
    end

    set -l directive (string sub --start 2 $__tt_perform_completion_once_result[-1])
    __tt_debug "Directive is: $directive"

    set -l shellCompDirectiveKeepOrder 32
    set -l keeporder (math (math --scale 0 $directive / $shellCompDirectiveKeepOrder) % 2)
    __tt_debug "Keeporder is: $keeporder"

    if test $keeporder -ne 0
        __tt_debug "This does require order preservation"
        return 0
    end

    __tt_debug "This doesn't require order preservation"
    return 1
end


# This function does two things:
# - Obtain the completions and store them in the global __tt_comp_results
# - Return false if file completion should be performed
function __tt_prepare_completions
    __tt_debug ""
    __tt_debug "========= starting completion logic =========="

    # Start fresh
    set --erase __tt_comp_results

    __tt_perform_completion_once
    __tt_debug "Completion results: $__tt_perform_completion_once_result"

    if test -z "$__tt_perform_completion_once_result"
        __tt_debug "No completion, probably due to a failure"
        # Might as well do file completion, in case it helps
        return 1
    end

    set -l directive (string sub --start 2 $__tt_perform_completion_once_result[-1])
    set --global __tt_comp_results $__tt_perform_completion_once_result[1..-2]

    __tt_debug "Completions are: $__tt_comp_results"
    __tt_debug "Directive is: $directive"

    set -l shellCompDirectiveError 1
    set -l shellCompDirectiveNoSpace 2
    set -l shellCompDirectiveNoFileComp 4
    set -l shellCompDirectiveFilterFileExt 8
    set -l shellCompDirectiveFilterDirs 16

    if test -z "$directive"
        set directive 0
    end

    set -l compErr (math (math --scale 0 $directive / $shellCompDirectiveError) % 2)
    if test $compErr -eq 1
        __tt_debug "Received error directive: aborting."
        # Might as well do file completion, in case it helps
        return 1
    end

    set -l filefilter (math (math --scale 0 $directive / $shellCompDirectiveFilterFileExt) % 2)
    set -l dirfilter (math (math --scale 0 $directive / $shellCompDirectiveFilterDirs) % 2)
    if test $filefilter -eq 1; or test $dirfilter -eq 1
        __tt_debug "File extension filtering or directory filtering not supported"
        # Do full file completion instead
        return 1
    end

    set -l nospace (math (math --scale 0 $directive / $shellCompDirectiveNoSpace) % 2)
    set -l nofiles (math (math --scale 0 $directive / $shellCompDirectiveNoFileComp) % 2)

    __tt_debug "nospace: $nospace, nofiles: $nofiles"

    # If we want to prevent a space, or if file completion is NOT disabled,
    # we need to count the number of valid completions.
    # To do so, we will filter on prefix as the completions we have received
    # may not already be filtered so as to allow fish to match on different
    # criteria than the prefix.
    if test $nospace -ne 0; or test $nofiles -eq 0
        set -l prefix (commandline -t | string escape --style=regex)
        __tt_debug "prefix: $prefix"

        set -l completions (string match -r -- "^$prefix.*" $__tt_comp_results)
        set --global __tt_comp_results $completions
        __tt_debug "Filtered completions are: $__tt_comp_results"

        # Important not to quote the variable for count to work
        set -l numComps (count $__tt_comp_results)
        __tt_debug "numComps: $numComps"

        if test $numComps -eq 1; and test $nospace -ne 0
            # We must first split on \t to get rid of the descriptions to be
            # able to check what the actual completion will be.
            # We don't need descriptions anyway since there is only a single
            # real completion which the shell will expand immediately.
            set -l split (string split --max 1 \t $__tt_comp_results[1])

            # Fish won't add a space if the completion ends with any
            # of the following characters: @=/:.,
            set -l lastChar (string sub -s -1 -- $split)
            if not string match -r -q "[@=/:.,]" -- "$lastChar"
                # In other cases, to support the "nospace" directive we trick the shell
                # by outputting an extra, longer completion.
                __tt_debug "Adding second completion to perform nospace directive"
                set --global __tt_comp_results $split[1] $split[1].
                __tt_debug "Completions are now: $__tt_comp_results"
            end
        end

        if test $numComps -eq 0; and test $nofiles -eq 0
            # To be consistent with bash and zsh, we only trigger file
            # completion when there are no other completions
            __tt_debug "Requesting file completion"
            return 1
        end
    end

    return 0
end

# Since Fish completions are only loaded once the user triggers them, we trigger them ourselves
# so we can properly delete any completions provided by another script.
# Only do this if the program can be found, or else fish may print some errors; besides,
# the existing completions will only be loaded if the program can be found.
if type -q "tt"
    # The space after the program name is essential to trigger completion for the program
    # and not completion of the program name itself.
    # Also, we use '> /dev/null 2>&1' since '&>' is not supported in older versions of fish.
    complete --do-complete "tt " > /dev/null 2>&1
end

# Remove any pre-existing completions for the program since we will be handling all of them.
complete -c tt -e

# this will get called after the two calls below and clear the $__tt_perform_completion_once_result global
complete -c tt -n '__tt_clear_perform_completion_once_result'
# The call to __tt_prepare_completions will setup __tt_comp_results
# which provides the program's completion choices.
# If this doesn't require order preservation, we don't use the -k flag
complete -c tt -n 'not __tt_requires_order_preservation && __tt_prepare_completions' -f -a '$__tt_comp_results'
# otherwise we use the -k flag
complete -k -c tt -n '__tt_requires_order_preservation && __tt_prepare_completions' -f -a '$__tt_comp_results'

# This file should be sourced by the main tt fish completion script,
# or fish_rocks.fish should be in a directory included in $fish_complete_path.
# For standalone testing, you might need to source fish_rocks.fish explicitly:
# source (dirname (status --current-filename))/fish_rocks.fish

# Helper to check if the command is 'tt rocks ...'
function __fish_is_tt_rocks
    set -l tokens (commandline -opc)
    if test (count $tokens) -ge 2; and test "$tokens[1]" = "tt"; and test "$tokens[2]" = "rocks"
        return 0
    end
    return 1
end

# Helper to check if the command is 'tt rocks admin ...'
function __fish_is_tt_rocks_admin
    set -l tokens (commandline -opc)
    if test (count $tokens) -ge 3; and test "$tokens[1]" = "tt"; and test "$tokens[2]" = "rocks"; and test "$tokens[3]" = "admin"
        return 0
    end
    return 1
end

# Completion for 'rocks' itself after 'tt'
# Condition: current command is 'tt' and we are at the second token.
complete -c tt -n 'set -l tkns (commandline -opc); test (count $tkns) -eq 1 -a "$tkns[1]" = "tt"' -f -a "rocks" -d "LuaRocks package manager"

# Completion for 'admin' after 'tt rocks'
# Condition: current command is 'tt rocks' and we are at the third token.
complete -c tt -n 'set -l tkns (commandline -opc); test (count $tkns) -eq 2 -a "$tkns[1]" = "tt" -a "$tkns[2]" = "rocks"' -f -a "(__fish_rocks_commands)" -d "Rocks command" # Offer all rocks commands, including admin

# Completion for 'tt rocks admin ...' subcommands and options
# Condition: current command starts with 'tt rocks admin'
complete -c tt -n '__fish_is_tt_rocks_admin' -f -a '(__fish_rocks_admin_complete)'

# Completion for 'tt rocks ...' subcommands and options (excluding admin path if already handled)
# Condition: current command starts with 'tt rocks' AND NOT 'tt rocks admin'
complete -c tt -n '__fish_is_tt_rocks; and not __fish_is_tt_rocks_admin' -f -a '(__fish_rocks_complete)'
# Fish completions for tt rocks

# Global options for 'rocks'
function __fish_rocks_global_options
    echo -e "-h\tShow this help message and exit"
    echo -e "--help\tShow this help message and exit"
    echo -e "--version\tShow version info and exit"
    echo -e "--dev\tEnable the sub-repositories in rocks servers for rockspecs of in-development versions"
    echo -e "--server\tFetch rocks/rockspecs from this server (takes priority over config file)"
    echo -e "--from\tFetch rocks/rockspecs from this server (takes priority over config file)"
    echo -e "--only-server\tFetch rocks/rockspecs from this server only (overrides any entries in the config file)"
    echo -e "--only-from\tFetch rocks/rockspecs from this server only (overrides any entries in the config file)"
    echo -e "--only-sources\tRestrict downloads to paths matching the given URL"
    echo -e "--only-sources-from\tRestrict downloads to paths matching the given URL"
    echo -e "--namespace\tSpecify the rocks server namespace to use"
    echo -e "--lua-dir\tWhich Lua installation to use"
    echo -e "--lua-version\tWhich Lua version to use"
    echo -e "--tree\tWhich tree to operate on"
    echo -e "--to\tWhich tree to operate on"
    echo -e "--local\tUse the tree in the user's home directory"
    echo -e "--global\tUse the system tree when local_by_default is true"
    echo -e "--no-project\tDo not use project tree even if running from a project folder"
    echo -e "--verbose\tDisplay verbose output of commands executed"
    echo -e "--timeout\tTimeout on network operations, in seconds"
    echo -e "--project-tree\tSpecify project tree"
    echo -e "--pack-binary-rock\tPack binary rock"
    echo -e "--branch\tSpecify branch"
    echo -e "--sign\tSign the rock"
end

# Options for 'rocks build' and 'rocks make'
function __fish_rocks_build_make_options
    __fish_rocks_global_options
    echo -e "--only-deps\tInstall only the dependencies of the rock"
    echo -e "--deps-only\tInstall only the dependencies of the rock"
    echo -e "--pin\tPin exact dependencies or create a luarocks.lock file"
    echo -e "--no-install\tDo not install the rock"
    echo -e "--no-doc\tInstall the rock without its documentation"
    echo -e "--keep\tDo not remove previously installed versions of the rock"
    echo -e "--force\tForce removal of previously installed versions"
    echo -e "--force-fast\tLike --force, but performs a forced removal without reporting dependency issues"
    echo -e "--verify\tVerify signature of the rockspec or src.rock being built"
    echo -e "--check-lua-versions\tReport if rock is available for another Lua version"
    echo -e "--no-manifest\tSkip creating/updating the manifest"
    echo -e "--chdir\tSpecify a source directory of the rock"
    echo -e "--deps-mode\tHow to handle dependencies (all one order none)"
    echo -e "--nodeps\tIgnore dependencies"
end

# Options for 'rocks config'
function __fish_rocks_config_options
    __fish_rocks_global_options
    echo -e "--scope\tThe scope indicates which config file should be rewritten (system user project)"
    echo -e "--unset\tDelete the key from the configuration file"
    echo -e "--json\tOutput as JSON"
    echo -e "--lua-incdir\tConfigure Lua include directory"
    echo -e "--lua-libdir\tConfigure Lua library directory"
    echo -e "--lua-ver\tConfigure Lua version"
    echo -e "--system-config\tPath to system config"
    echo -e "--user-config\tPath to user config"
    echo -e "--rock-trees\tConfigure rock trees"
end

# Options for 'rocks doc'
function __fish_rocks_doc_options
    __fish_rocks_global_options
    echo -e "--home\tOpen the home page of project"
    echo -e "--list\tList documentation files only"
    echo -e "--porcelain\tProduce machine-friendly output"
end

# Options for 'rocks download'
function __fish_rocks_download_options
    __fish_rocks_global_options
    echo -e "--all\tDownload all files if there are multiple matches"
    echo -e "--source\tDownload .src.rock if available"
    echo -e "--rockspec\tDownload .rockspec if available"
    echo -e "--arch\tDownload rock for a specific architecture"
    echo -e "--check-lua-versions\tReport if rock is available for another Lua version"
end

# Options for 'rocks install'
function __fish_rocks_install_options
    __fish_rocks_global_options
    echo -e "--keep\tDo not remove previously installed versions"
    echo -e "--force\tForce removal of previously installed versions"
    echo -e "--force-fast\tLike --force, but performs a forced removal"
    echo -e "--only-deps\tInstall only the dependencies of the rock"
    echo -e "--deps-only\tInstall only the dependencies of the rock"
    echo -e "--no-doc\tInstall the rock without its documentation"
    echo -e "--verify\tVerify signature of the rockspec or src.rock being built"
    echo -e "--check-lua-versions\tReport if rock is available for another Lua version"
    echo -e "--deps-mode\tHow to handle dependencies (all one order none)"
    echo -e "--nodeps\tIgnore dependencies"
    echo -e "--no-manifest\tSkip creating/updating the manifest"
    echo -e "--pin\tCreate a luarocks.lock file"
end

# Options for 'rocks lint'
function __fish_rocks_lint_options
    __fish_rocks_global_options
end

# Options for 'rocks list'
function __fish_rocks_list_options
    __fish_rocks_global_options
    echo -e "--outdated\tList only rocks for which there is a higher version available"
    echo -e "--porcelain\tProduce machine-friendly output"
end

# Options for 'rocks make_manifest' or 'make-manifest'
function __fish_rocks_make_manifest_options
    __fish_rocks_global_options
    echo -e "--local-tree\tDo not write versioned versions of the manifest file"
    echo -e "--deps-mode\tHow to handle dependencies (all one order none)"
    echo -e "--nodeps\tIgnore dependencies"
end

# Options for 'rocks new_version' or 'new-version'
function __fish_rocks_new_version_options
    __fish_rocks_global_options
    echo -e "--dir\tOutput directory for the new rockspec"
    echo -e "--tag\tNew SCM tag"
end

# Options for 'rocks pack'
function __fish_rocks_pack_options
    __fish_rocks_global_options
    # --sign is already global
end

# Options for 'rocks purge'
function __fish_rocks_purge_options
    __fish_rocks_global_options
    echo -e "--old-versions\tKeep the highest-numbered version of each rock and remove others"
    echo -e "--force\tForce removal if it would break dependencies"
    echo -e "--force-fast\tLike --force, but performs a forced removal"
end

# Options for 'rocks remove'
function __fish_rocks_remove_options
    __fish_rocks_global_options
    echo -e "--force\tForce removal if it would break dependencies"
    echo -e "--force-fast\tPerform a forced removal without reporting dependency issues"
    echo -e "--deps-mode\tHow to handle dependencies (all one order none)"
    echo -e "--nodeps\tIgnore dependencies"
end

# Options for 'rocks search'
function __fish_rocks_search_options
    __fish_rocks_global_options
    echo -e "--source\tReturn only rockspecs and source rocks"
    echo -e "--binary\tReturn only pure Lua and binary rocks"
    echo -e "--all\tList all contents of the server suitable to this platform"
    echo -e "--porcelain\tReturn a machine readable format"
end

# Options for 'rocks show'
function __fish_rocks_show_options
    __fish_rocks_global_options
    echo -e "--home\tShow home page of project"
    echo -e "--modules\tShow all modules provided by the package"
    echo -e "--deps\tShow packages the package depends on"
    echo -e "--build-deps\tShow build-only dependencies for the package"
    echo -e "--test-deps\tShow dependencies for testing the package"
    echo -e "--rockspec\tShow the full path of the rockspec file"
    echo -e "--mversion\tShow the package version"
    echo -e "--rock-tree\tShow local tree where rock is installed"
    echo -e "--rock-namespace\tShow rock namespace"
    echo -e "--rock-dir\tShow data directory of the installed rock"
    echo -e "--rock-license\tShow rock license"
    echo -e "--issues\tShow URL for project's issue tracker"
    echo -e "--labels\tList the labels of the rock"
    echo -e "--porcelain\tProduce machine-friendly output"
end

# Options for 'rocks test'
function __fish_rocks_test_options
    __fish_rocks_global_options
    echo -e "--prepare\tOnly install dependencies needed for testing, do not run test"
    echo -e "--test-type\tSpecify the test suite type manually"
end

# Options for 'rocks unpack'
function __fish_rocks_unpack_options
    __fish_rocks_global_options
    echo -e "--force\tUnpack files even if the output directory already exists"
    echo -e "--check-lua-versions\tReport if rock is available for another Lua version"
end

# Options for 'rocks which'
function __fish_rocks_which_options
    __fish_rocks_global_options
end

# Options for 'rocks write_rockspec' or 'write-rockspec'
function __fish_rocks_write_rockspec_options
    __fish_rocks_global_options
    echo -e "--output\tWrite the rockspec with the given filename"
    echo -e "--license\tA license string, such as \"MIT/X11\""
    echo -e "--summary\tA short one-line description summary"
    echo -e "--detailed\tA longer description string"
    echo -e "--homepage\tProject homepage"
    echo -e "--lua-versions\tSupported Lua versions (e.g., 5.1 5.1,5.2)"
    echo -e "--rockspec-format\tRockspec format version, such as \"1.0\""
    echo -e "--tag\tTag to use"
    echo -e "--lib\tA comma-separated list of libraries to link to"
end

# Main completion function for 'rocks'
function __fish_rocks_complete
    set -l tokens (commandline -opc)
    set -l current_word (commandline -ct)
    set -l prev_word
    if test (count $tokens) -gt 1; set prev_word $tokens[-1]; end

    set -l sub_command
    # tt rocks <sub_command> ...
    # tokens[1] is tt, tokens[2] is rocks
    if test (count $tokens) -ge 3
        set sub_command $tokens[3]
    end

    if string match -q -- "-*" "$current_word"
        switch "$sub_command"
            case "build" "make"
                __fish_rocks_build_make_options
            case "config"
                __fish_rocks_config_options
            case "doc"
                __fish_rocks_doc_options
            case "download"
                __fish_rocks_download_options
            case "install"
                __fish_rocks_install_options
            case "lint"
                __fish_rocks_lint_options
            case "list"
                __fish_rocks_list_options
            case "make_manifest" "make-manifest"
                __fish_rocks_make_manifest_options
            case "new_version" "new-version"
                __fish_rocks_new_version_options
            case "pack"
                __fish_rocks_pack_options
            case "purge"
                __fish_rocks_purge_options
            case "remove"
                __fish_rocks_remove_options
            case "search"
                __fish_rocks_search_options
            case "show"
                __fish_rocks_show_options
            case "test"
                __fish_rocks_test_options
            case "unpack"
                __fish_rocks_unpack_options
            case "which"
                __fish_rocks_which_options
            case "write_rockspec" "write-rockspec"
                __fish_rocks_write_rockspec_options
            case "*" # No subcommand or unknown, or completing option for 'rocks' itself
                __fish_rocks_global_options
        end
    else if test -z "$sub_command" -o "$prev_word" = "rocks" # Completing the subcommand itself
        __fish_rocks_commands
    else
        # Argument completion (e.g., file paths, specific values)
        switch "$prev_word"
            case "--server" "--from" "--only-server" "--only-from" \
                 "--only-sources" "--only-sources-from" "--namespace" \
                 "--lua-dir" "--lua-version" "--tree" "--to" "--timeout" \
                 "--project-tree" "--branch" "--chdir" "--arch" \
                 "--dir" "--tag" "--output" "--license" "--summary" \
                 "--detailed" "--homepage" "--rockspec-format" "--lib" \
                 "--test-type"
                __fish_complete_path "$current_word"
            case "--deps-mode"
                echo -e "all\tAll dependencies"
                echo -e "one\tOne level of dependencies"
                echo -e "order\tOrder dependencies"
                echo -e "none\tNo dependencies"
            case "--scope"
                echo -e "system\tSystem scope"
                echo -e "user\tUser scope"
                echo -e "project\tProject scope"
            case "--lua-versions"
                for v in 5.1 5.2 5.3 5.4 "5.1,5.2" "5.2,5.3" "5.3,5.4" "5.1,5.2,5.3" "5.2,5.3,5.4" "5.1,5.2,5.3,5.4"
                    echo -e "$v\tLua version(s) $v"
                end
            case "*"
                 # Default: complete rock names or file paths
                 # For commands like install, build, show, etc.
                 __fish_complete_path "$current_word"
        end
    end
end

# Commands for 'rocks'
function __fish_rocks_commands
    echo -e "help\tShow help for commands"
    echo -e "admin\tLuaRocks repository administration interface"
    echo -e "build\tBuild/compile a rock"
    echo -e "config\tQuery information about the LuaRocks configuration"
    echo -e "doc\tShow documentation for an installed rock"
    echo -e "download\tDownload a specific rock file from a rocks server"
    echo -e "install\tInstall a rock"
    echo -e "lint\tCheck syntax of a rockspec"
    echo -e "list\tList currently installed rocks"
    echo -e "make\tCompile package in current directory using a rockspec"
    echo -e "make_manifest\tCompile a manifest file for a repository"
    echo -e "make-manifest\t(alias for make_manifest)"
    echo -e "new_version\tAuto-write a rockspec for a new version of a rock"
    echo -e "new-version\t(alias for new_version)"
    echo -e "pack\tCreate a rock, packing sources or binaries"
    echo -e "purge\tRemove all installed rocks from a tree"
    echo -e "remove\tUninstall a rock"
    echo -e "search\tQuery the LuaRocks servers"
    echo -e "show\tShow information about an installed rock"
    echo -e "test\tRun the test suite in the current directory"
    echo -e "unpack\tUnpack the contents of a rock"
    echo -e "which\tTell which file corresponds to a given module name"
    echo -e "write_rockspec\tWrite a template for a rockspec file"
    echo -e "write-rockspec\t(alias for write_rockspec)"
end

# Global options for 'rocks admin'
function __fish_rocks_admin_global_options
    echo -e "-h\tShow this help message and exit"
    echo -e "--help\tShow this help message and exit"
    echo -e "--version\tShow version info and exit"
    echo -e "--dev\tEnable the sub-repositories in rocks servers"
    echo -e "--server\tFetch rocks/rockspecs from this server"
    echo -e "--from\t(alias for --server)"
    echo -e "--only-server\tFetch rocks/rockspecs from this server only"
    echo -e "--only-from\t(alias for --only-server)"
    echo -e "--only-sources\tRestrict downloads to paths matching the given URL"
    echo -e "--only-sources-from\t(alias for --only-sources)"
    echo -e "--namespace\tSpecify the rocks server namespace to use"
    echo -e "--lua-dir\tWhich Lua installation to use"
    echo -e "--lua-version\tWhich Lua version to use"
    echo -e "--tree\tWhich tree to operate on"
    echo -e "--to\t(alias for --tree)"
    echo -e "--local\tUse the tree in the user's home directory"
    echo -e "--global\tUse the system tree when local_by_default is true"
    echo -e "--no-project\tDo not use project tree"
    echo -e "--verbose\tDisplay verbose output"
    echo -e "--timeout\tTimeout on network operations, in seconds"
    echo -e "--project-tree\tSpecify project tree"
end

# Options for 'rocks admin add'
function __fish_rocks_admin_add_options
    __fish_rocks_admin_global_options
    echo -e "--no-refresh\tDo not refresh the local cache"
    echo -e "--index\tProduce an index.html file for the manifest"
end

# Options for 'rocks admin make_manifest' or 'make-manifest'
function __fish_rocks_admin_make_manifest_options
    __fish_rocks_admin_global_options
    echo -e "--local-tree\tDo not write versioned versions of the manifest file"
    echo -e "--deps-mode\tHow to handle dependencies (all one order none)"
    echo -e "--nodeps\tIgnore dependencies"
end

# Options for 'rocks admin refresh_cache' or 'refresh-cache'
function __fish_rocks_admin_refresh_cache_options
    __fish_rocks_admin_global_options
    # --from is already global for admin
end

# Options for 'rocks admin remove'
function __fish_rocks_admin_remove_options
    __fish_rocks_admin_global_options
    echo -e "--no-refresh\tDo not refresh the local cache"
end

# Main completion function for 'rocks admin'
function __fish_rocks_admin_complete
    set -l tokens (commandline -opc)
    set -l current_word (commandline -ct)
    set -l prev_word
    if test (count $tokens) -gt 1; set prev_word $tokens[-1]; end

    set -l sub_command_admin
    # tt rocks admin <sub_command_admin> ...
    # tokens[1] is tt, tokens[2] is rocks, tokens[3] is admin
    if test (count $tokens) -ge 4
        set sub_command_admin $tokens[4]
    end

    if string match -q -- "-*" "$current_word"
        switch "$sub_command_admin"
            case "add"
                __fish_rocks_admin_add_options
            case "make_manifest" "make-manifest"
                __fish_rocks_admin_make_manifest_options
            case "refresh_cache" "refresh-cache"
                __fish_rocks_admin_refresh_cache_options
            case "remove"
                __fish_rocks_admin_remove_options
            case "*" # No subcommand or unknown, or completing option for 'admin' itself
                __fish_rocks_admin_global_options
        end
    else if test -z "$sub_command_admin" -o "$prev_word" = "admin" # Completing the admin subcommand itself
        __fish_rocks_admin_commands
    else
        # Argument completion for admin subcommands
        switch "$prev_word"
            case "--server" "--from" "--only-server" "--only-from" \
                 "--only-sources" "--only-sources-from" "--namespace" \
                 "--lua-dir" "--lua-version" "--tree" "--to" "--timeout" \
                 "--project-tree"
                __fish_complete_path "$current_word"
            case "--deps-mode"
                 echo -e "all\tAll dependencies"
                 echo -e "one\tOne level of dependencies"
                 echo -e "order\tOrder dependencies"
                 echo -e "none\tNo dependencies"
            case "*" # Default file completion for args of admin commands (e.g. rockspec files for add/remove)
                 __fish_complete_path "$current_word"
        end
    end
end

# Commands for 'rocks admin'
function __fish_rocks_admin_commands
    echo -e "help\tShow help for commands"
    echo -e "add\tAdd a rock or rockspec to a rocks server"
    echo -e "make_manifest\tCompile a manifest file for a repository"
    echo -e "make-manifest\t(alias for make_manifest)"
    echo -e "refresh_cache\tRefresh local cache of a remote rocks server"
    echo -e "refresh-cache\t(alias for refresh_cache)"
    echo -e "remove\tRemove a rock or rockspec from a rocks server"
end
